@@ -73,6 +73,7 @@ class FakeVimHandler;
 class WebSocketServerService;
 class QOwnNotesMarkdownTextEdit;
 class CommandBar;
+class NoteFilePathLabel;
 struct TagHeader;
 
 // forward declaration because of "xxx does not name a type"
@@ -117,6 +118,8 @@ class MainWindow : public QMainWindow {
 
     void triggerStartupMenuAction();
 
+    void reloadOpenAiControls();
+
     void setCurrentNoteText(QString text);
 
     void setCurrentNote(Note note, bool updateNoteText = true, bool updateSelectedNote = true,
@@ -528,6 +531,12 @@ class MainWindow : public QMainWindow {
 
     void onWorkspaceComboBoxCurrentIndexChanged(int index);
 
+    void onAiBackendComboBoxCurrentIndexChanged(int index);
+
+    void onAiModelComboBoxCurrentIndexChanged(int index);
+
+    void onAiModelGroupChanged(QAction *action);
+
     void on_actionRemove_current_workspace_triggered();
 
     void on_actionRename_current_workspace_triggered();
@@ -648,7 +657,7 @@ class MainWindow : public QMainWindow {
 
     void onLanguageChanged(QAction *action);
 
-    void onBackendChanged(QAction *action);
+    void onSpellBackendChanged(QAction *action);
 
     void on_actionManage_dictionaries_triggered();
 
@@ -702,6 +711,8 @@ class MainWindow : public QMainWindow {
 
     void on_actionMove_down_in_tag_list_triggered();
 
+    void on_actionEnable_AI_toggled(bool arg1);
+
    public:
     /** Settings access **/
     static bool isInDistractionFreeMode();
@@ -755,6 +766,7 @@ class MainWindow : public QMainWindow {
     NoteHistory noteHistory;
     QHash<int, NoteHistoryItem> noteBookmarks;
     QPushButton *_updateAvailableButton;
+    NoteFilePathLabel *_noteFilePathLabel;
     QLabel *_noteEditLineNumberLabel;
     QPushButton *_readOnlyButton;
     QPushButton *_leaveDistractionFreeModeButton;
@@ -763,6 +775,7 @@ class MainWindow : public QMainWindow {
     QToolBar *_customActionToolbar;
     QToolBar *_insertingToolbar;
     QToolBar *_encryptionToolbar;
+    QToolBar *_aiToolbar;
     QToolBar *_windowToolbar;
     QToolBar *_quitToolbar;
     bool _noteViewIsRegenerated;
@@ -803,6 +816,8 @@ class MainWindow : public QMainWindow {
     QWidget *_logDockTitleBarWidget;
     QWidget *_scriptingDockTitleBarWidget;
     QComboBox *_workspaceComboBox;
+    QComboBox *_aiBackendComboBox;
+    QComboBox *_aiModelComboBox;
     QFrame *_noteFolderDockWidgetFrame;
     bool _useNoteFolderButtons;
     bool _noteFolderDockWidgetWasVisible;
@@ -824,6 +839,7 @@ class MainWindow : public QMainWindow {
     WebAppClientService *_webAppClientService;
     QActionGroup *_languageGroup;
     QActionGroup *_spellBackendGroup;
+    QActionGroup *_aiModelGroup;
     bool _brokenTagNoteLinksRemoved = false;
 
 #ifdef USE_QLITEHTML
@@ -1115,4 +1131,9 @@ class MainWindow : public QMainWindow {
     int getNoteTabIndex(int noteId) const;
     bool startAutoReadOnlyModeIfEnabled();
     void updateActionUiEnabled();
+    void buildAiToolbarAndActions();
+    void generateAiBackendComboBox();
+    void generateAiModelComboBox();
+    void generateAiModelMainMenu();
+    void aiModelMainMenuSetCurrentItem();
 };
