@@ -43,6 +43,7 @@ QT_WARNING_DISABLE_GCC("-Wmismatched-new-delete")
 #include <QInputDialog>
 #include <QMessageBox>
 
+#include "openaiservice.h"
 #include "widgets/qownnotesmarkdowntextedit.h"
 #endif
 
@@ -302,7 +303,17 @@ void ScriptingService::initComponents() {
 /**
  * Reloads the engine
  */
-void ScriptingService::reloadEngine() { reloadScriptComponents(); }
+void ScriptingService::reloadEngine() {
+    reloadScriptComponents();
+
+#ifndef INTEGRATION_TESTS
+    MainWindow *mainWindow = MainWindow::instance();
+    if (mainWindow != nullptr) {
+        // Reload the OpenAI controls, so that the scripts can add custom backends
+        mainWindow->reloadOpenAiControls();
+    }
+#endif
+}
 
 /**
  * Returns the registered script variables
@@ -583,6 +594,38 @@ QStringList ScriptingService::callAutocompletionHook() const {
     return results;
 }
 
+/**
+ * Calls the openAiBackendsHook function for all script components
+ * This function is called when OpenAI service is initialized
+ *
+ * @return QList of QJsonObject with the backend information
+ */
+QList<QVariantMap> ScriptingService::callOpenAiBackendsHook() const {
+    QMapIterator<int, ScriptComponent> i(_scriptComponents);
+    QList<QVariantMap> results;
+
+    while (i.hasNext()) {
+        i.next();
+        ScriptComponent scriptComponent = i.value();
+        QVariant result;
+
+        if (methodExistsForObject(scriptComponent.object, QStringLiteral("openAiBackendsHook()"))) {
+            QMetaObject::invokeMethod(scriptComponent.object, "openAiBackendsHook",
+                                      Q_RETURN_ARG(QVariant, result));
+            // Convert the returned value to QVariantList
+            QVariantList resultList = result.toList();
+
+            for (const QVariant &item : resultList) {
+                QVariantMap itemMap = item.toMap();
+
+                results.append(itemMap);
+            }
+        }
+    }
+
+    return results;
+}
+
 /**
  * Calls the insertingFromMimeDataHook function for an object
  */
@@ -1332,6 +1375,23 @@ QString ScriptingService::noteTextEditCurrentWord(bool withPreviousCharacters) {
 #endif
 }
 
+/**
+ * Reads the current block in the note text edit
+ *
+ * @return
+ */
+QString ScriptingService::noteTextEditCurrentBlock() {
+    MetricsService::instance()->sendVisitIfEnabled(QStringLiteral("scripting/") %
+                                                   QString(__func__));
+
+#ifndef INTEGRATION_TESTS
+    MainWindow *mainWindow = MainWindow::instance();
+    return mainWindow != nullptr ? mainWindow->activeNoteTextEdit()->currentBlock() : QString();
+#else
+    return {};
+#endif
+}
+
 /**
  * Tags the current note with a tag named tagName
  *
@@ -1384,6 +1444,23 @@ QString ScriptingService::downloadUrlToString(const QUrl &url) {
     return Utils::Misc::downloadUrl(url);
 }
 
+/**
+ * QML wrapper to use the AI Completer
+ *
+ * @param prompt
+ * @return {QString} the result of the completer
+ */
+QString ScriptingService::aiComplete(const QString &prompt) {
+#ifndef INTEGRATION_TESTS
+    MetricsService::instance()->sendVisitIfEnabled(QStringLiteral("scripting/") %
+                                                   QString(__func__));
+
+    return OpenAiService::instance()->complete(prompt);
+#else
+    Q_UNUSED(prompt)
+#endif
+}
+
 /**
  * QML wrapper to download an url to the media folder and returning the media
  * url or the Markdown image text of the media relative to the current note
@@ -1989,7 +2066,10 @@ QString ScriptingService::inputDialogGetItem(const QString &title, const QString
                                                    QString(__func__));
 
 #ifndef INTEGRATION_TESTS
-    return QInputDialog::getItem(nullptr, title, label, items, current, editable);
+    bool ok;
+    QString result = QInputDialog::getItem(nullptr, title, label, items, current, editable, &ok);
+
+    return ok ? result : QStringLiteral("");
 #else
     Q_UNUSED(title)
     Q_UNUSED(label)
@@ -2014,7 +2094,36 @@ QString ScriptingService::inputDialogGetText(const QString &title, const QString
                                                    QString(__func__));
 
 #ifndef INTEGRATION_TESTS
-    return QInputDialog::getText(nullptr, title, label, QLineEdit::Normal, text);
+    bool ok;
+    QString result = QInputDialog::getText(nullptr, title, label, QLineEdit::Normal, text, &ok);
+
+    return ok ? result : QStringLiteral("");
+#else
+    Q_UNUSED(title)
+    Q_UNUSED(label)
+    Q_UNUSED(text)
+    return QString();
+#endif
+}
+
+/**
+ * Opens an input dialog with a multi-line text edit
+ *
+ * @param title {QString} title of the dialog
+ * @param label {QString} label text of the dialog
+ * @param text {QString} text in the dialog (optional)
+ * @return
+ */
+QString ScriptingService::inputDialogGetMultiLineText(const QString &title, const QString &label,
+                                                      const QString &text) {
+    MetricsService::instance()->sendVisitIfEnabled(QStringLiteral("scripting/") %
+                                                   QString(__func__));
+
+#ifndef INTEGRATION_TESTS
+    bool ok;
+    QString result = QInputDialog::getMultiLineText(nullptr, title, label, text, &ok);
+
+    return ok ? result : QStringLiteral("");
 #else
     Q_UNUSED(title)
     Q_UNUSED(label)
