@@ -27,8 +27,8 @@
 #include <QDebug>
 #include <QDialogButtonBox>
 #include <QDockWidget>
-#include <QListWidget>
 #include <QLineEdit>
+#include <QListWidget>
 #include <QMessageBox>
 #include <QPlainTextEdit>
 #include <QProcess>
@@ -1221,10 +1221,31 @@ void Utils::Gui::fixDarkModeIcons(QWidget *widget) {
         return;
     }
 
-    foreach(QLineEdit *lineEdit, widget->findChildren<QLineEdit *>()) {
+    foreach (QLineEdit *lineEdit, widget->findChildren<QLineEdit *>()) {
         auto action = lineEdit->findChild<QAction *>("_q_qlineeditclearaction");
         if (action) {
             action->setIcon(QIcon(QStringLiteral(":/images/cleartext-dark.svg")));
         }
     }
 }
+
+QAction *Utils::Gui::findActionByData(QMenu *menu, const QVariant &data) {
+    // Iterate through all actions in the menu
+    for (QAction *action : menu->actions()) {
+        // Check if the action's data matches the desired data
+        if (action->data() == data) {
+            return action;
+        }
+
+        // Check if the action is a submenu
+        if (QMenu *subMenu = action->menu()) {
+            // Recursively search in the submenu
+            QAction *foundAction = findActionByData(subMenu, data);
+            if (foundAction) {
+                return foundAction;
+            }
+        }
+    }
+    // Return nullptr if no matching action is found
+    return nullptr;
+}
