@@ -623,6 +623,12 @@ QString QOwnNotesMarkdownTextEdit::currentWord(bool withPreviousCharacters) cons
     return text.trimmed();
 }
 
+QString QOwnNotesMarkdownTextEdit::currentBlock() const {
+    QTextCursor cursor = textCursor();
+    QTextBlock currentBlock = cursor.block();
+    return currentBlock.text();
+}
+
 /**
  * Tries to find words that start with the current word in the note text edit
  *
